#include <PalmOS.h>

#include "PalmOSGUI.h"
#include "FrodoResources.h"
#include "PalmHelper.h"
#include "PalmHardware.h"


//***********************************************************
// Procedure: DeviceHasNVFS
//
// Parameter: -
//
// Returnval: 1 if device has NVFS
//
// Description: Check if hardware has NVFS. 
//***********************************************************
Boolean DeviceHasNVFS(void)
{
  UInt32 error;
  UInt32 lFlags = 0;
  
  error = FtrGet(sysFtrCreator, sysFtrNumDmAutoBackup, &lFlags);
  if(lFlags == 1)
    return true;
  else
    return false;
}

//***********************************************************
// Procedure: DeviceHas5Way
//
// Parameter: -
//
// Returnval: 1 if device has five way buttons
//
// Description: Check if hardware has 5 way buttons. 
//***********************************************************
Boolean DeviceHas5Way(void)
{
	UInt32 error;
	UInt32 lFlags;
	
	// Check feature: documented but not available
  //	if(!error && (lFlags & sysFtrNumUIHardwareHas5Way))
  error = FtrGet('fway', 0, &lFlags); // Found this in internet
	if(!error && lFlags )
		return 1;

	return 0;
}

//***********************************************************
// Procedure: CheckPrerequisites
//
// Parameter: -
//
// Returnval: Errorcode or 0 if hardware matches requirements
//
// Description: Check if hardware matches requirements. 
//***********************************************************
UInt32 CheckPrerequisites(void)
{
	UInt32 w,h,d, error=0;
	Boolean c;

	// OS5+?
	error = FtrGet(sysFtrCreator, sysFtrNumROMVersion, &d);
	if (error || d < sysMakeROMVersion(5,0,0,0,0))
	{
		ShowErrorResLong(StringErrWrongOSVersion, d);
		return 2;
	}
	// ARM processor?
	error = FtrGet(sysFileCSystem, sysFtrNumProcessorID, &d);
	if (error || !sysFtrNumProcessorIsARM(d))
	{
		ShowErrorResLong(StringErrNoARM, d);
		return 7;
	}
	// Hires support?
	error = FtrGet(sysFtrCreator, sysFtrNumWinVersion, &d);
	if (error || d < 4)
	{
	  ShowErrorResLong(StringErrNoHIRES, d);
		return 3;
	}
	// Hires?
	WinScreenGetAttribute(winScreenDensity, &d);
	if (d == kDensityDouble)
		is320 = 1;

	// Wider than 320?
	error=WinScreenGetAttribute(winScreenWidth, &screenW);
	if (error)
	{
		ShowErrorRes(StringErrNoScreenWidth);
		return 9;
	}
	if (screenW<320 && d == kDensityDouble)
	{
	  ShowErrorResLong(StringErrHiresWithout320, screenW);
		return 10;
	}
	// Higher than 320?
	error=WinScreenGetAttribute(winScreenHeight, &screenH);
	if (error)
	{
		ShowErrorRes(StringErrNoScreenHeight);
		return 11;
	}
	// Actually, right now I'm demanding 320 because I'm lazy and can't
	// bother to turn of the on-screen keyboard if there isn't room for it
	if (screenH<320 && d == kDensityDouble)
	{
		ShowErrorResLong(StringErrScreenHeightToSmall, screenH);
		return 12;
	}

	// Some optional features
	// Native 256 color mode?
	c = true;
	error = WinScreenMode(winScreenModeGetSupportedDepths, &w, &h, &d, &c);
	if (!error && (d & 128))
		can256=true;

	// Sampled sound?

	// Does NOT WORK AT ALL on 5.0 Tungsten T v1 with pre-production OS 5.0
	// (ie. my Palm device!)
	if (!(error = FtrGet(sysFileCSoundMgr, sndFtrIDVersion, &d)))
		canSound = true;
	FtrGet(sysFtrCreator, sysFtrNumOEMDeviceID, &d);
	if(d == palmDeviceID_TT) // Tungsten T. 
		canSound = true;
		
	return 0;
}
