//
//  PRC.java
//  C64Palmer
//
//  Created by Johan Forsberg on Thu Aug 28 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

import java.io.*;
import java.util.*;

public class PDB {
    ArrayList items=new ArrayList();
    D64Handler d64=new D64Handler();
    T64Handler t64=new T64Handler();
    ZipHandler zip=new ZipHandler();
    
    public PDB() {
	clear();
    }
    
    java.util.List addFile(File f) {
        ArrayList al=new ArrayList();
        if (f.isDirectory()) {
            // Recursive add
            File files[]=f.listFiles(new C64FileFilter());
            if (files==null) return al;
            
            for (int i=0; i<files.length; i++)
                al.addAll(addFile(files[i]));
            return al;
        }
        
        String n=f.getName();
	if (n==null || n.length()<4)
		return al;
	String ext=n.substring(n.length()-4);
        if (ext.equalsIgnoreCase(".zip")) {
            List zl=zip.handle(f);
            if (zl==null) return al;
            Iterator i=zl.iterator();
            while (i.hasNext()) {
                PDBItem p=(PDBItem)i.next();
                items.add(p);
                al.add(p);
            }
            return al;
        }
        
        PDBItem it=null;
        if (ext.equalsIgnoreCase(".d64"))
            it=d64.handle(f);
        else if (ext.equalsIgnoreCase(".t64"))
            it=t64.handle(f);
                
        if (it!=null) {
            items.add(it);
            al.add(it);
        }
        return al;
    }
    
    void clear() {
        items.clear();
    }
    
    void saveAs(File f) throws IOException {
	int i,j,k;
	Random r=new Random();
	String name="Frodo import DB"+r.nextInt(1000000);
	FileOutputStream fos=new FileOutputStream(f);

	// Database header
	// Name
	for (i=0; i<32; i++) {
		if (i<name.length())
			fos.write((byte) name.charAt(i));
		else
			fos.write(0);
	}
	// Attributes, just backup for now
	fos.write(0x00); fos.write(0x08);
	// Version
	fos.write(0x01); fos.write(0x00);
	
	GregorianCalendar epoch=new GregorianCalendar(1904, 1, 1, 12, 0, 0);
	GregorianCalendar now=new GregorianCalendar();
	long millis=now.getTime().getTime()-epoch.getTime().getTime();
	int sec=(int) millis/1000;
	
	// Creation date
	fos.write((sec>>24)&0xff); fos.write((sec>>16)&0xff);
	fos.write((sec>>8)&0xff);  fos.write(sec&0xff);
    
	// Modification date
	fos.write((sec>>24)&0xff); fos.write((sec>>16)&0xff);
	fos.write((sec>>8)&0xff);  fos.write(sec&0xff);
    
	// Backup date
	fos.write(0); fos.write(0); fos.write(0); fos.write(0);
	
	// Modification number
	fos.write(0); fos.write(0); fos.write(0); fos.write(1);
	
	// app info offset
	fos.write(0); fos.write(0); fos.write(0); fos.write(0);
	
	// Sort info offset
	fos.write(0); fos.write(0); fos.write(0); fos.write(0);
	
	// type
	fos.write('D'); fos.write('A'); fos.write('T'); fos.write('A');
	
	// Creator
	fos.write('F'); fos.write('r'); fos.write('o'); fos.write('d');
	
	// Unique ID seed??
	fos.write(0); fos.write(0); fos.write(0); fos.write(0);
	
	// Record List
	PDBItem l[] = new PDBItem[items.size()];
	items.toArray(l);
	int parts=0;
	
	for (i=0; i<l.length; i++)
		parts+=l[i].elements();

	int size[] = new int [parts];
	int offset[] = new int[parts];
	int hdrSize=0x48+6+parts*0x8;

	for (i=0,k=0; i<l.length; i++) {
		for (j=0; j<l[i].elements(); j++, k++) {
			size[k]=l[i].elementSize(j);
			if (k==0)
				offset[k]=hdrSize;
			else
				offset[k]=offset[k-1]+size[k-1];
		}
	}
	
	// Next record list ID
	fos.write(0); fos.write(0); fos.write(0); fos.write(0);
	
	// Number of items
	fos.write((parts>>8)&0xff); fos.write(parts&0xff);
	
	// Directory entries
	for (i=0; i<parts; i++) {
		// File offset
		fos.write((offset[i]>>24)&0xff);
		fos.write((offset[i]>>16)&0xff);
		fos.write((offset[i]>>8)&0xff);
		fos.write(offset[i]&0xff);
		// Attributes
		fos.write(0);
		// unique id
		fos.write(i&0xff); fos.write((i>>8)&0xff); fos.write(i&0xff);
	}
	
	// The entries themselves
	for (i=0; i<l.length; i++)
		l[i].write(fos);
	fos.close();
    }
}
