/*
 *  PrefsState.h - Global preferences, as a struct
 *
 *  Frodo (C) 1994-1997,2002 Christian Bauer
 */

#ifndef _PREFSSTATE_H
#define _PREFSSTATE_H

// The options used in the Palm version, as a struct.
// This struct needs to be aligned exactly the same way with the ARM and the
// 68k. I use int32 exclusively to be on the safe side.

// Drive types
enum {
        DRVTYPE_DIR,    // 1541 emulation in host file system
        DRVTYPE_D64,    // 1541 emulation in .d64 file
        DRVTYPE_T64     // 1541 emulation in .t64 file
};


#define JOYPEN_GRAFFITI 1
#define JOYPEN_SCREEN   2

// Pixels on screen:	12345678
// Pixels in int32:	04030201 08070605
#define SCREENMODE_256		0L
#define SCREENMODE_256_BUFFERED	1L

// Options for saving to SD card
#define SAVE2SD_NEVER   0L
#define SAVE2SD_ASK     1L
#define SAVE2SD_ALWAYS  2L

#define C64COMMANDS_MAX 5
#define C64COMMANDS_TITLELEN 16
#define C64COMMANDS_MAXLEN 64


struct PrefsState {
	int32 NormalCycles;		// CPU cycles in normal raster lines
	int32 BadLineCycles;		// CPU cycles in Bad Lines
	int32 CIACycles;		// CIA timer ticks per raster line
	int32 FloppyCycles;		// 1541 CPU cycles per line

	int32 AdaptiveFrameSkip;	// Skip frames as nessecary
	int32 SkipFrames;		// Draw every n-th frame
	int32 LimitSpeed;		// Limit speed to 100%

	int32 DriveType;		// Type of drive 8..11

	char DrivePath[32];		// "Path" for drive 8..11
  int32 DriveCard;
  int32 DriveIndex;
  
	int32 SIDType;			// SID emulation type

	int32 SpritesOn;		// Sprite display is on
	int32 SpriteCollisions;		// Sprite collision detection is on
	int32 Joystick1On;		// Joystick connected to port 1 of host
	int32 Joystick2On;		// Joystick connected to port 2 of host
	int32 JoystickSwap;		// Swap joysticks 1<->2
	int32 FastReset;		// Skip RAM test on reset
	int32 CIAIRQHack;		// Write to CIA ICR clears IRQ
	int32 Emul1541Proc;		// Enable processor-level 1541 emulation
	int32 SIDFilters;		// Emulate SID filters

	int32 BordersOn;
	int32 SingleCycleEmulation;
	int32 SIDOn;
};


// PalmOS only prefs beneath this line
struct PreferencesPalm
{
	int32 ScreenMode;
	int32 Volume;			// Sound volume (1024==100%)
	int32 JoyPen;			// JoyPen mode
	uint32 KeyUp;			// Key for up
	uint32 KeyLeft;			// Key for left
	uint32 KeyRight;			// Key for right
	uint32 KeyDown;			// Key for down
	uint32 KeyFire;			// Key for fire button
	
	int16 SaveToSDCard; // Flag, how to save to sd card
	
	uint32 KeyMapping[66]; // Map C64 key to Palm key
	unsigned char colorTable[16][4]; // Current color table
	char lastSnapshotName[32];
	int32 lastSnapshotIndex;
	int16 paused;
	int16 DisplayTrackNum;	// Display track number of disk drive 8 if 1541 emulation is active
  char C64CommandTitle[C64COMMANDS_MAX][C64COMMANDS_TITLELEN];
  char C64Command[C64COMMANDS_MAX][C64COMMANDS_MAXLEN];
  uint32 SoundBufferSize;

	int16 SpeedLimit;   // Current speed limit: 10% - 100%
};

#endif
